require 'json'
require 'aduki'

module PassengerInfo
  VERSION = "0.0.1"

  class ConnectionTime < Aduki::Initializable
    attr_accessor :local, :relative, :relative_timestamp, :timestamp
  end

  class RequestInfo < Aduki::Initializable
    attr_accessor :protocol, :host, :path, :pid, :method, :number, :connected_at

    def self.from_client_data data
      req = data["current_request"]
      session = req["session"] || { }
      built = new number: data["number"], pid: session["pid"], method: req["method"], protocol: (req["flags"]["https"] ? "https" : "http" ), host: req["host"], path: req["path"], connected_at: ConnectionTime.new(data["connected_at"])
      raise "wrong type: #{built.connected_at.class}" unless built.connected_at.is_a?(PassengerInfo::ConnectionTime)
      built
    end

    def to_s
      "PID #{pid.to_s.rjust(6)} #{method} #{protocol}://#{host}#{path} : connected at #{connected_at.local} (#{connected_at.relative})"
    end
  end

  def self.from_server_data hsh
    hsh["active_clients"].values.map { |c| PassengerInfo::RequestInfo.from_client_data c }
  end

  def self.from_server_status txt
    hsh          = JSON.load txt
    (1..hsh["threads"]).map { |n| hsh["thread#{n}"] }.map { |i| from_server_data i }.flatten.sort_by &:number
  end
end
